@@ -1,4 +1,4 @@
-// Copyright (C) 2021  Tassilo Horn <tsdh@gnu.org>
+// Copyright (C) 2021-2022  Tassilo Horn <tsdh@gnu.org>
 //
 // This program is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the Free
@@ -23,6 +23,10 @@ use std::path as p;
 use std::process as proc;
 
 pub fn get_swayr_socket_path() -> String {
+    // We prefer checking the env variable instead of
+    // directories::BaseDirs::new().unwrap().runtime_dir().unwrap() because
+    // directories errors if the XDG_RUNTIME_DIR isn't set or set to a relative
+    // path which actually works fine for sway & swayr.
     let xdg_runtime_dir = std::env::var("XDG_RUNTIME_DIR");
     let wayland_display = std::env::var("WAYLAND_DISPLAY");
     format!(
@@ -30,14 +34,14 @@ pub fn get_swayr_socket_path() -> String {
         match xdg_runtime_dir {
             Ok(val) => val,
             Err(_e) => {
-                eprintln!("Couldn't get XDG_RUNTIME_DIR!");
+                log::error!("Couldn't get XDG_RUNTIME_DIR!");
                 String::from("/tmp")
             }
         },
         match wayland_display {
             Ok(val) => val,
             Err(_e) => {
-                eprintln!("Couldn't get WAYLAND_DISPLAY!");
+                log::error!("Couldn't get WAYLAND_DISPLAY!");
                 String::from("unknown")
             }
         }
@@ -92,7 +96,7 @@ fn desktop_entries() -> Vec<Box<p::Path>> {
 fn find_icon(icon_name: &str, icon_dirs: &[String]) -> Option<Box<p::Path>> {
     let p = p::Path::new(icon_name);
     if p.is_file() {
-        println!("(1) Icon name '{}' -> {}", icon_name, p.display());
+        log::debug!("(1) Icon name '{}' -> {}", icon_name, p.display());
         return Some(p.to_path_buf().into_boxed_path());
     }
 
@@ -102,7 +106,7 @@ fn find_icon(icon_name: &str, icon_dirs: &[String]) -> Option<Box<p::Path>> {
             pb.push(icon_name.to_owned() + "." + ext);
             let icon_file = pb.as_path();
             if icon_file.is_file() {
-                println!(
+                log::debug!(
                     "(2) Icon name '{}' -> {}",
                     icon_name,
                     icon_file.display()
@@ -112,13 +116,13 @@ fn find_icon(icon_name: &str, icon_dirs: &[String]) -> Option<Box<p::Path>> {
         }
     }
 
-    println!("(3) No icon for name {}", icon_name);
+    log::debug!("(3) No icon for name {}", icon_name);
     None
 }
 
 lazy_static! {
     static ref WM_CLASS_OR_ICON_RX: regex::Regex =
-        regex::Regex::new("(StartupWMClass|Icon)=(.+)").unwrap();
+        regex::Regex::new(r"(StartupWMClass|Icon)=(.+)").unwrap();
     static ref REV_DOMAIN_NAME_RX: regex::Regex =
         regex::Regex::new(r"^(?:[a-zA-Z0-9-]+\.)+([a-zA-Z0-9-]+)$").unwrap();
 }
@@ -183,7 +187,7 @@ fn get_app_id_to_icon_map(
         }
     }
 
-    println!(
+    log::debug!(
         "Desktop entries to icon files ({} entries):\n{:#?}",
         map.len(),
         map
@@ -225,12 +229,13 @@ fn test_icon_stuff() {
 
 pub trait DisplayFormat {
     fn format_for_display(&self, config: &cfg::Config) -> String;
+    fn get_indent_level(&self) -> usize;
 }
 
 pub fn select_from_menu<'a, 'b, TS>(
     prompt: &'a str,
     choices: &'b [TS],
-) -> Option<&'b TS>
+) -> Result<&'b TS, String>
 where
     TS: DisplayFormat + Sized,
 {
@@ -273,7 +278,7 @@ where
             .as_mut()
             .expect("Failed to open the menu program's stdin");
         let input = strs.join("\n");
-        println!("Menu program {} input:\n{}", menu_exec, input);
+        //log::debug!("Menu program {} input:\n{}", menu_exec, input);
         stdin
             .write_all(input.as_bytes())
             .expect("Failed to write to the menu program's stdin");
@@ -283,5 +288,5 @@ where
     let choice = String::from_utf8_lossy(&output.stdout);
     let mut choice = String::from(choice);
     choice.pop(); // Remove trailing \n from choice.
-    map.get(&choice).copied()
+    map.get(&choice).copied().ok_or(choice)
 }
